`timescale 1 ps / 1 ps

module a_tb ();

// Constants
parameter	REF_CLK_PERIOD 			= 6400;		//156.25MHz
parameter	MGMT_CLK_PERIOD 		= 10000;		//100MHz
parameter	counter_width_0 = 32;
parameter	counter_width_1 = 64;
parameter	COUNT_VALUE = 55;	// Specify the number of slip needed
parameter	PMA_WIDTH = 64;		// Specify your PMA width settings
parameter	RECONFIG_ADD_WIDTH = 11;

// General purpose registers
reg clock_tb;
reg mgmt_blk_clock_tb;
reg reset_tb;
reg [PMA_WIDTH - 1:0] tx_parallel_data_tb;

reg [39:0] tx_bitslip_tb;
reg rx_set_locktodata_tb;
reg rx_set_locktoref_tb;
reg rx_seriallpbken_tb;
reg tx_std_polinv_tb;

reg pll_select_tb;

// Bit slip functions
reg		[counter_width_0 - 1:0]	counter_0;
reg		[counter_width_1 - 1:0]	counter_1;
reg		start_count;
reg		start_count_1;
reg		[counter_width_0-1:0]	count_val;
reg rx_bitslip_tb;


// Reconfig management input
reg reconfig_mgmt_read_tb;
reg reconfig_mgmt_write_tb;
reg [RECONFIG_ADD_WIDTH - 1:0] reconfig_mgmt_address_tb;
reg [31:0] reconfig_mgmt_writedata_tb;

reg stream_req_tb;
reg cfg_sel_tb;

// PRBS verifier
reg rx_10g_prbs_err_clr_tb;
wire rx_10g_prbs_done_tb;
wire rx_10g_prbs_err_tb;


// Reconfig management
wire reconfig_busy_tb;
wire reconfig_mgmt_waitrequest_tb;
wire [31:0] reconfig_mgmt_readdata_tb;

wire	count_done;

wire	stream_busy_tb;

// Wires
wire tx_ready_tb;
wire rx_ready_tb;
wire pll_locked_tb;
wire tx_serial_data_tb;
wire rx_serial_data_tb;

wire rx_is_lockedtoref_tb;
wire rx_is_lockedtodata_tb;
wire tx_clkout_tb;
wire rx_clkout_tb;
wire [PMA_WIDTH - 1:0] rx_parallel_data_tb;

wire rx_std_signaldetect_tb;

wire [1:0] tx_serial_clk0_tb;
wire tx_serial_clk_tb;

// External loopback TX -> RX
assign		rx_serial_data_tb = tx_serial_data_tb;

// Connect the pll serial clock
//assign tx_serial_clk0_tb[0] = tx_serial_clk_tb;
//assign tx_serial_clk0_tb[1] = tx_serial_clk_tb;

// Start the test

initial
begin

//	tx_parallel_data_tb = {128'h0BC, 128'h0BC};
	tx_parallel_data_tb = 64'h0ABCD;

       end

initial
begin
	
	start_count = 1'b0;
	start_count_1 = 1'b0;
	count_val = 32'h00000000;
	reset_tb 		= 1'b0;
	pll_select_tb = 1'b0;
	stream_req_tb = 1'b0;
	cfg_sel_tb	= 1'b0;


	// rx_set_locktoref_tb & rx_set_locktodata_tb = 0 -> CDR auto lock
	// mode
	rx_set_locktoref_tb	= 1'b0;
	rx_set_locktodata_tb	= 1'b0;
	rx_seriallpbken_tb	= 1'b0;
	rx_bitslip_tb		= 1'b0;
	tx_std_polinv_tb	= 1'b0;
	reconfig_mgmt_read_tb	= 1'b0;
	reconfig_mgmt_write_tb	= 1'b0;
	reconfig_mgmt_address_tb	= 7'b0;
	reconfig_mgmt_writedata_tb	= 32'b0;
	rx_10g_prbs_err_clr_tb  = 1'b0;


	#50000
	reset_tb = 1'b1;
	#100000
	reset_tb = 1'b0;



	#10000000
	start_count = 1'b1;
	count_val = COUNT_VALUE;

	#20000000
	start_count_1 = 1'b1;


end


////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  Clocks

//reference clock
initial
begin
  clock_tb = 1;
  forever
    #(REF_CLK_PERIOD/2) clock_tb = !clock_tb;
end			

// MGMT reference clock
initial
begin
  mgmt_blk_clock_tb = 1;
  forever
    #(MGMT_CLK_PERIOD/2) mgmt_blk_clock_tb = !mgmt_blk_clock_tb;
end


////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
//
// #################### multiple bit slip function ################################



assign	count_done = (counter_0 == count_val)? 1'b1 : 1'b0; 

always @(posedge clock_tb or posedge reset_tb)
	if (reset_tb)
		counter_0 = {counter_width_0{1'b0}};
	else if (count_done)
		begin
			counter_0 = {counter_width_0{1'b0}};
			start_count = 1'b0;
		end
	else if (start_count)
		begin
			// Applying bit slips
			#100000
			@(negedge clock_tb) begin
				rx_bitslip_tb			= 1'b1;
			end
			
			#100000
			@(negedge clock_tb) begin
				rx_bitslip_tb			= 1'b0;
			end
					
			counter_0 = counter_0 + 1'b1;
		end

// #################### Incremental counter function ################################


always @(posedge tx_clkout_tb or posedge reset_tb)
	if (reset_tb)
		counter_1 = {counter_width_1{1'b0}};
	else if (start_count_1)
		begin
			tx_parallel_data_tb = counter_1;
			counter_1 = counter_1 + 1'b1;
		end

// ################################################################

a 	DUT	(
	.clock			(mgmt_blk_clock_tb),
	.pll_refclk0			(clock_tb),
	.rx_serial_data			(rx_serial_data_tb),
	.tx_parallel_data		(tx_parallel_data_tb),
	.tx_ready			(tx_ready_tb),
	.rx_ready			(rx_ready_tb),
	.rx_pma_clkslip		(rx_bitslip_tb),
	.rx_clkout		(rx_clkout_tb),
	.tx_clkout		(tx_clkout_tb),
	.tx_serial_data			(tx_serial_data_tb),
	.rx_is_lockedtoref			(rx_is_lockedtoref_tb),
	.rx_parallel_data		(rx_parallel_data_tb),
	.reset				(reset_tb),
	.pll_select			(pll_select_tb)
);

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

// Task to perform write
task write32;
input [RECONFIG_ADD_WIDTH - 1:0] reconfig_mgmt_address;
input [31:0] reconfig_mgmt_writedata;

begin
	@(posedge mgmt_blk_clock_tb) begin

		// reconfig address for logical channel
		reconfig_mgmt_address_tb = reconfig_mgmt_address;
		reconfig_mgmt_writedata_tb = reconfig_mgmt_writedata;
	end

	@(posedge mgmt_blk_clock_tb) begin

		// Write to register
		reconfig_mgmt_write_tb = 1'b1;
	end

	// Reset the reconfig write signal
	@(posedge mgmt_blk_clock_tb) begin

		reconfig_mgmt_write_tb = 1'b0;
	end
	
end
endtask

// Task to perform read
task read32;
input [RECONFIG_ADD_WIDTH - 1:0] reconfig_mgmt_address;

begin
@(posedge mgmt_blk_clock_tb) begin

		// reconfig address of busy register
		reconfig_mgmt_address_tb = reconfig_mgmt_address;
	end

	@(posedge mgmt_blk_clock_tb) begin

		// read register
		reconfig_mgmt_read_tb = 1'b1;
	end

	// Reset the reconfig read signal
	@(posedge mgmt_blk_clock_tb) begin

		reconfig_mgmt_read_tb = 1'b0;
	end
end
	
endtask

endmodule
